////////////////////////////////////////////////////////////////////////////////
// File         : graph.cpp
// Subsystem    : Graph
// Component    : Graph
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : implementation of the class Hnet
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 10.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
//     
////////////////////////////////////////////////////////////////////////////////

#include "graph.h"

// ****************************** Graph ************************************

Graph::Graph(const Graph &g)
{
	AddCopyOf(g);
}

Graph& Graph::operator=(const Graph &g)
{
	if (this != &g)
	{
		DeleteAllNodes();
		AddCopyOf(g);
	}
	return *this;
}

int Graph::NumberOfNodes(void) const
{
	return nodes.size();
}

int Graph::NumberOfEdges(void) const
{
	return edges.size();
}

int Graph::NumberOfIngoingEdges( const Node &n) const
{
	int NumberOfIngoingEdges = 0;

	Edge edge = n.FirstIngoingEdge();

	while ( edge )
	{
		NumberOfIngoingEdges++;
		edge = edge.NextIngoingEdge();
	}

	return NumberOfIngoingEdges;
}

int Graph::NumberOfOutgoingEdges( const Node &n) const
{
	int NumberOfOutgoingEdges = 0;

	Edge edge = n.FirstOutgoingEdge();

	while ( edge )
	{
		NumberOfOutgoingEdges++;
		edge = edge.NextOutgoingEdge();
	}

	return NumberOfOutgoingEdges;
}



Node Graph::CreateNode(void)
{
	std::list<NodeData>::iterator		_n;
	std::list<std::list<NeighbourData> >::iterator	_neigh;
	nodes.push_back(NodeData( ));
	_n = nodes.end();
	--_n;

	// leere Vorgngerliste erzeugen:
	neighbours.push_back(std::list<NeighbourData>());
	_neigh = neighbours.end();
	--_neigh;
	_n->_inNeighbours = _neigh;

	// leere Nachfolgerliste erzeugen:
	neighbours.push_back(std::list<NeighbourData>());
	_neigh = neighbours.end();
	--_neigh;
	_n->_outNeighbours = _neigh;

	Node n(this, _n);
	n.NewAttributeVector( &nodeAttributeVectorTable );
	_n->attr = n.GetAttributeVectorPtr(); 
	return n;
}

void Graph::Delete(Node &n)
{
	if (n)
	{
		unsigned int i;
		Edge e;
		std::vector<Edge> ve;

		for (e=n.FirstIngoingEdge(); e; e=e.NextIngoingEdge())
			ve.push_back(e);
		for (e=n.FirstOutgoingEdge(); e; e=e.NextOutgoingEdge())
			ve.push_back(e);
		for (i=0; i<ve.size(); ++i)
			Delete(ve[i]);
		neighbours.erase(n._n->_inNeighbours);
		neighbours.erase(n._n->_outNeighbours);
		nodes.erase(n._n);
		n.DelAttributeVector();
		n._g = 0;
	}
}



Edge Graph::CreateEdge(const Node &s, const Node &t)
{
	if (s && t && (s != t))
	{
		std::list<EdgeData>::iterator _edge;
		edges.push_back(EdgeData());
		_edge = edges.end();
		--_edge;
		std::list<NeighbourData>::iterator _neigh;

		// Verknpfung der Kante mit Quellknoten:
		s._n->_outNeighbours->push_back(NeighbourData());
		_neigh = s._n->_outNeighbours->end();
		--_neigh;
		_edge->_inNeighbours = s._n->_outNeighbours;
		_edge->_sourceNeighbour = _neigh;
		_neigh->_n = Cast2NodeDataPtr( s._n );
		_neigh->_e = Cast2EdgeDataPtr( _edge );

		// Verknpfung der Kante mit Zielknoten:
		t._n->_inNeighbours->push_back(NeighbourData());
		_neigh = t._n->_inNeighbours->end();
		--_neigh;
		_edge->_outNeighbours = t._n->_inNeighbours;
		_edge->_targetNeighbour = _neigh;
		_neigh->_n = Cast2NodeDataPtr( t._n );
		_neigh->_e = Cast2EdgeDataPtr( _edge );

		Edge e(this, _edge);;
		e.NewAttributeVector( &edgeAttributeVectorTable );
		_edge->attr = e.GetAttributeVectorPtr();

		return e;
	}
	else
		return Edge();
}


Edge Graph::CreateEdgeSingle( const Node &s, const Node &t) 
{
	Edge e = ExistingEdge( s, t );

	if (!e )
		return CreateEdge( s, t );
	else
		return e; // not added if edge with this nodes exists already
}


void Graph::Delete(Edge &e)
{
	if (e)
	{
		e._e->_inNeighbours->erase(e._e->_sourceNeighbour);
		e._e->_outNeighbours->erase(e._e->_targetNeighbour);
		edges.erase(e._e);
		e.DelAttributeVector();
		e._g = 0;
	}
}


Node Graph::FirstNode(void) const
{
	if (nodes.empty())
		return Node();
	else
		return Node(this, nodes.begin());
}


Edge Graph::FirstEdge(void) const
{
	if (edges.empty())
		return Edge();
	else
		return Edge(this, edges.begin());
}


Node Graph::LastNode(void) const
{
	if (nodes.empty())
		return Node();
	else
		return Node(this, --(nodes.end()));
}


Edge Graph::LastEdge(void) const
{
	if (edges.empty())
		return Edge();
	else
		return Edge(this, --(edges.end()));
}


Node Graph::AddCopyOf(const Graph &g)
{
	NodeAttribute< int > n2id;
	std::vector<Node>	id2n;
        Node		n;
        int		i;

	for (n=g.FirstNode(), i=0; n; n=n.NextNode(), ++i)
	{
		n2id[n] = i;
		id2n.push_back(CreateNode());
	}

	for (Edge e=g.FirstEdge(); e; e=e.NextEdge())
		CreateEdge(id2n[n2id[e.SourceNode()]], id2n[n2id[e.TargetNode()]]);

	return id2n[0]; // first new node
}


void Graph::DeleteAllNodes(void)
{
	Node n;
	while ( n = FirstNode() )
		Delete( n );
}


Edge Graph::ExistingEdge( const Node &n1, const Node &n2 ) const
{
	Edge e = n1.FirstOutgoingEdge();

	while ( e )
	{
		if ( e.TargetNode() == n2 )
			return e; // edge exists already
		e = e.NextOutgoingEdge();
	}

	return Edge();
}

Edge Graph::AddTransitiveHull( const Node &n1, const Node &n2)
{
	Edge e = LastEdge();

	std::vector< Node > in;
	std::vector< Node > out;

	Edge inEdge, outEdge;

	inEdge = n1.FirstIngoingEdge();

	while ( inEdge )
	{
		in.push_back( inEdge.SourceNode() );
		inEdge = inEdge.NextIngoingEdge();
	}

	outEdge = n2.FirstOutgoingEdge();

	while ( outEdge )
	{
		out.push_back( outEdge.TargetNode() );
		outEdge = outEdge.NextOutgoingEdge();
	}

	int i, j;

/*
	for( i=0; i<in.size(); i++ )
		for( j=0; j<out.size(); j++)
			CreateEdgeSingle( in[ i ], out[j] );
*/

// This next block realizes the same edges as the double loop in comment before.
// But the sequence or neighborhood is different and should speed up
// the colored tree algorithm for the trnsitive complement,
// because the first edge from n-th entry goes to n-th exit.

	if (in.size() <= out.size())
	{
		for( i=0; i<in.size(); i++ )
		{
			for( j=i; j<out.size(); j++)
				CreateEdgeSingle( in[ i ], out[j] );

			for( j=0; j<i; j++)
				CreateEdgeSingle( in[ i ], out[j] );
		}
	}
	else
		for( j=0; j<out.size(); j++ )
		{
			for( i=j; i<in.size(); i++)
				CreateEdgeSingle( in[ i ], out[j] );

			for( i=0; i<j; i++)
				CreateEdgeSingle( in[ i ], out[j] );
		}

	return e.NextEdge();
}

	
Edge Graph::AddTransitiveHull( const Node &n ) 
{ 
	return AddTransitiveHull( n, n ); 
}


Edge Graph::AddTransitiveHull( const Edge &e )
{ 
	return AddTransitiveHull( e.SourceNode(), e.TargetNode() ); 
}

	
void Graph::DeleteTransitiveHull( const Node &n1, const Node &n2 ) //delete all edges from predecestors of n1 to ancestors of n2
{
	Edge inEdge, outEdge, e, ee;

	inEdge = n1.FirstIngoingEdge();

	while ( inEdge )
	{
		e = inEdge.SourceNode().FirstOutgoingEdge();

		while ( e )
		{
			outEdge = n2.FirstOutgoingEdge();

			while ( outEdge )
			{
				if ( outEdge.TargetNode() == e.TargetNode() )
				{
					ee = e.NextOutgoingEdge();
					Delete( e );
					outEdge = Edge();  // invalid edge to stop while-loop
				}
				else
					outEdge = outEdge.NextOutgoingEdge();
			}

			if ( e )
				e = e.NextOutgoingEdge();
			else 
				e = ee; // e was deleted
		}

		inEdge = inEdge.NextIngoingEdge();
	}
}


void Graph::DeleteTransitiveHull( const Node &n )
{ 
	DeleteTransitiveHull( n, n ); 
}


void Graph::DeleteTransitiveHull( Edge &e )	
{ 
	Graph::DeleteTransitiveHull( e.SourceNode(), e.TargetNode() ); 
}


Edge Graph::DeleteTransitiveHullReverse( Edge &edge )	// { similar to DeleteTransitiveHull( e.TargetNode(), e.SourceNode() ); };
{ 
	std::vector< Node > in;
	std::vector< Node >::iterator _itIn;
	Edge inEdge, outEdge, enext;

	// store sources
	inEdge = edge.TargetNode().FirstIngoingEdge();
	while ( inEdge )
	{
		in.push_back( inEdge.SourceNode() );
		inEdge = inEdge.NextIngoingEdge();
	}


	// delete edges
	for( _itIn = in.begin(); _itIn != in.end(); ++_itIn )
	{
		outEdge = _itIn->FirstOutgoingEdge();
		while ( outEdge )
		{
			enext = outEdge.NextOutgoingEdge();
			if ( outEdge != edge)
				Delete( outEdge );
			outEdge = enext;
		}
	}

	enext = edge.NextEdge();
	Delete( edge );

	return enext;
}


	
Node Graph::AddStar( const Node &n1, const Node &n2 ) //add a node and connect to all predecestors of n1 and to all ancestors of n2
{
	Node n = CreateNode();

	Edge e = n1.FirstIngoingEdge();

	while ( e )
	{
		CreateEdge( e.SourceNode(), n );
		e = e.NextIngoingEdge();
	}
	
	e = n2.FirstOutgoingEdge();

	while ( e )
	{
		CreateEdge( n, e.TargetNode() );
		e = e.NextOutgoingEdge();
	}

	return n;
}


Node Graph::AddStarBetween( const Node &n ) //create new predecestors and ancestors an connect to old ones
{
	Node nn;

	std::vector< Edge > in;
	std::vector< Edge > out;
	std::vector< Edge >::iterator _itIn;
	std::vector< Edge >::iterator _itOut;

	Edge inEdge, outEdge;

	// store old ingoing edges
	inEdge = n.FirstIngoingEdge();
	while ( inEdge )
	{
		in.push_back( inEdge );
		inEdge = inEdge.NextIngoingEdge();
	}

	// store old outgoing edges
	outEdge = n.FirstOutgoingEdge();
	while ( outEdge )
	{
		out.push_back( outEdge );
		outEdge = outEdge.NextOutgoingEdge();
	}


	// add nodes and ingoing edges
	for( _itIn = in.begin(); _itIn != in.end(); ++_itIn )
	{
		nn = CreateNode();
		CreateEdge( nn, n );
		CreateEdge( _itIn->SourceNode(), nn );
	}

	// add nodes and outgoing edges
	for( _itOut = out.begin(); _itOut != out.end(); ++_itOut)
	{
		nn = CreateNode();
		CreateEdge( n, nn );
		CreateEdge( nn, _itOut->TargetNode() );
	}

	// del old ingoing edges
	for( _itIn = in.begin(); _itIn != in.end(); ++_itIn )
	{
		Delete( *_itIn );
	}

	// del old outgoing edges
	for( _itOut = out.begin(); _itOut != out.end(); ++_itOut)
	{
		Delete( *_itOut );
	}

	return n;
}



Node Graph::AddStar( const Edge &e )
{ 
	return AddStar( e.SourceNode(), e.TargetNode() ); 
}


void Graph::DeleteStarBetween( const Node &n1, const Node &n2 ) //delete all predecestors/ancessors and connect directly to n1/n2 
{ 
	std::vector< Edge > in;
	std::vector< Edge > out;
	std::vector< Edge >::iterator _itIn;
	std::vector< Edge >::iterator _itOut;

	Edge inEdge, outEdge, ee;
	Node n;

	// store old ingoing edges
	inEdge = n1.FirstIngoingEdge();
	while ( inEdge )
	{
		in.push_back( inEdge );
		inEdge = inEdge.NextIngoingEdge();
	}

	// store old outgoing edges
	outEdge = n2.FirstOutgoingEdge();
	while ( outEdge )
	{
		out.push_back( outEdge );
		outEdge = outEdge.NextOutgoingEdge();
	}

	// del predessors
	for( _itIn = in.begin(); _itIn != in.end(); ++_itIn )
	{
		n = _itIn->SourceNode();
		ee = n.FirstIngoingEdge();
		while ( ee ) // overbridge ingoing edges
		{
			CreateEdge( ee.SourceNode(), n1 );
			ee = ee.NextIngoingEdge();
		}

		ee = n.FirstOutgoingEdge();
		while ( ee ) // overbridge outgoing edges
		{
			CreateEdge( n1, ee.TargetNode() );
			ee = ee.NextOutgoingEdge();
		}

		Delete( n );
	}

	// del ancestors
	for( _itOut = out.begin(); _itOut != out.end(); ++_itOut)
	{
		n = _itOut->TargetNode();
		ee = n.FirstIngoingEdge();
		while ( ee ) // overbridge ingoing edges
		{
			CreateEdge( ee.SourceNode(), n2 );
			ee = ee.NextIngoingEdge();
		}

		ee = n.FirstOutgoingEdge();
		while ( ee ) // overbridge outgoing edges
		{
			CreateEdge( n2, ee.TargetNode() );
			ee = ee.NextOutgoingEdge();
		}

		Delete( n );
	}
 }


void Graph::DeleteStarBetween( const Node &n ) //delete all predecestors and ancessors and connect directly to n
{ 
	std::vector< Edge > in;
	std::vector< Edge > out;
	std::vector< Edge >::iterator _itIn;
	std::vector< Edge >::iterator _itOut;

	Edge inEdge, outEdge, ee;
	Node nn;

	// store old ingoing edges
	inEdge = n.FirstIngoingEdge();
	while ( inEdge )
	{
		in.push_back( inEdge );
		inEdge = inEdge.NextIngoingEdge();
	}

	// store old outgoing edges
	outEdge = n.FirstOutgoingEdge();
	while ( outEdge )
	{
		out.push_back( outEdge );
		outEdge = outEdge.NextOutgoingEdge();
	}

	// del predecessors
	for( _itIn = in.begin(); _itIn != in.end(); ++_itIn )
	{
		nn = _itIn->SourceNode();
		ee = nn.FirstIngoingEdge();
		if ( ee )
			CreateEdge( ee.SourceNode(), n );
		Delete( nn );
	}

	// del ancestors
	for( _itOut = out.begin(); _itOut != out.end(); ++_itOut)
	{
		nn = _itOut->TargetNode();
		ee = nn.FirstOutgoingEdge();
		if ( ee )
			CreateEdge( n, ee.TargetNode() );
		Delete( nn );
	}
}


void Graph::DeleteStarBetween( const Edge &e )	// { DeleteStarBetween( e.SourceNode(), e.TargetNode() ); };
{ 
	DeleteStarBetween( e.SourceNode(), e.TargetNode() ); 
}




Node Graph::CreateNodeCopy( const Node &n )	// add Node and copy edges
{
	Edge inEdge, outEdge;
	Node nn = CreateNode();

	// store old ingoing edges
	inEdge = n.FirstIngoingEdge();
	while ( inEdge )
	{
		CreateEdge( inEdge.SourceNode(), nn );
		inEdge = inEdge.NextIngoingEdge();
	}

	// store old outgoing edges
	outEdge = n.FirstOutgoingEdge();
	while ( outEdge )
	{
		CreateEdge( nn, outEdge.TargetNode() );
		outEdge = outEdge.NextOutgoingEdge();
	}

	return nn;
}


void Graph::IsolateNode( const Node &n )	// del ingoing and outgoing edges
{
	std::vector< Edge > in;
	std::vector< Edge > out;
	std::vector< Edge >::iterator _itIn;
	std::vector< Edge >::iterator _itOut;

	Edge inEdge, outEdge;

	// store old ingoing edges
	inEdge = n.FirstIngoingEdge();
	while ( inEdge )
	{
		in.push_back( inEdge );
		inEdge = inEdge.NextIngoingEdge();
	}

	// store old outgoing edges
	outEdge = n.FirstOutgoingEdge();
	while ( outEdge )
	{
		out.push_back( outEdge );
		outEdge = outEdge.NextOutgoingEdge();
	}

	// del ingoing edges
	for( _itIn = in.begin(); _itIn != in.end(); ++_itIn )
	{
		Delete( *_itIn );
	}

	// del outgoing edges
	for( _itOut = out.begin(); _itOut != out.end(); ++_itOut)
	{
		Delete( *_itOut );
	}
}



// ****************************** Node ************************************

Node Node::NextNode(void) const
{
	if (_g == 0)
		return Node();
	std::list<Graph::NodeData>::iterator _n2 = _n;
	++_n2;
	if (_n2 == _g->nodes.end())
		return Node();
	else
		return Node(_g, _n2);
}

Node Node::PreviousNode(void) const
{
	if (_g == 0)
		return Node();
	std::list<Graph::NodeData>::iterator _n2 = _n;
	--_n2;
	if (_n == _g->nodes.begin())
		return Node();
	else
		return Node(_g, _n2);
}

Edge Node::FirstIngoingEdge(void) const
{
	if (_g == 0)
		return Edge();
	if (_n->_inNeighbours->empty())
		return Edge();
	else
		return Edge(_g, Graph::Cast2EdgeDataRef( _n->_inNeighbours->front()._e) );
}

Edge Node::FirstOutgoingEdge(void) const
{
	if (_g == 0)
		return Edge();
	if (_n->_outNeighbours->empty())
		return Edge();
	else
		return Edge(_g, Graph::Cast2EdgeDataRef( _n->_outNeighbours->front()._e) );
}

Edge Node::LastIngoingEdge(void) const
{
	if (_g == 0)
		return Edge();
	if (_n->_inNeighbours->empty())
		return Edge();
	else
		return Edge(_g, Graph::Cast2EdgeDataRef( _n->_inNeighbours->back()._e) );
}

Edge Node::LastOutgoingEdge(void) const
{
	if (_g == 0)
		return Edge();
	if (_n->_outNeighbours->empty())
		return Edge();
	else
		return Edge(_g, Graph::Cast2EdgeDataRef( _n->_outNeighbours->back()._e) );
}

bool operator==(const Node &n1, const Node &n2)
{
	return (n1._g == n2._g) && (n1._n == n2._n);
}

bool operator!=(const Node &n1, const Node &n2)
{
	return !(n1 == n2);
}

Node::operator bool() const
{
	return _g != 0;
}

// ****************************** Edge ************************************

Edge Edge::NextEdge(void) const
{
	if (_g == 0)
		return Edge();
	std::list<Graph::EdgeData>::iterator _e2 = _e;
	++_e2;
	if (_e2 == _g->edges.end())
		return Edge();
	else
		return Edge(_g, _e2);
}

Edge Edge::PreviousEdge(void) const
{
	if (_g == 0)
		return Edge();
	std::list<Graph::EdgeData>::iterator _e2 = _e;
	--_e2;
	if (_e == _g->edges.begin())
		return Edge();
	else
		return Edge(_g, _e2);
}

Node Edge::SourceNode(void) const
{
	if (_g == 0)
		return Node();
	return Node(_g, Graph::Cast2NodeDataRef( _e->_sourceNeighbour->_n ));
}

Node Edge::TargetNode(void) const
{
	if (_g == 0)
		return Node();
	return Node(_g, Graph::Cast2NodeDataRef( _e->_targetNeighbour->_n ));
}

Edge Edge::NextIngoingEdge(void) const
{
	if (_g == 0)
		return Edge();
	std::list<Graph::NeighbourData>::iterator _e2 = _e->_targetNeighbour;
	++_e2;
	if (_e2 == _e->_outNeighbours->end())
		return Edge();
	else
		return Edge(_g, Graph::Cast2EdgeDataRef( _e2->_e ) );
}

Edge Edge::PreviousIngoingEdge(void) const
{
	if (_g == 0)
		return Edge();
	std::list<Graph::NeighbourData>::iterator _e2 = _e->_targetNeighbour;
	--_e2;
	if (_e->_targetNeighbour == _e->_outNeighbours->begin())
		return Edge();
	else
		return Edge(_g, Graph::Cast2EdgeDataRef( _e2->_e ) );
}

Edge Edge::NextOutgoingEdge(void) const
{
	if (_g == 0)
		return Edge();
	std::list<Graph::NeighbourData>::iterator _e2 = _e->_sourceNeighbour;
	++_e2;
	if (_e2 == _e->_inNeighbours->end())
		return Edge();
	else
		return Edge(_g, Graph::Cast2EdgeDataRef( _e2->_e ));
}

Edge Edge::PreviousOutgoingEdge(void) const
{
	if (_g == 0)
		return Edge();
	std::list<Graph::NeighbourData>::iterator _e2 = _e->_sourceNeighbour;
	--_e2;
	if (_e->_sourceNeighbour == _e->_inNeighbours->begin())
		return Edge();
	else
		return Edge(_g, Graph::Cast2EdgeDataRef( _e2->_e ));
}

bool operator==(const Edge &e1, const Edge &e2)
{
	return (e1._g == e2._g) && (e1._e == e2._e);
}

bool operator!=(const Edge &e1, const Edge &e2)
{
	return !(e1 == e2);
}

Edge::operator bool() const
{
	return _g != 0;
}

//////////////////////////////////////////////////////////////////////
void NodePath::Print( char *comment, NodeAttribute< int > &attribut ) // ?? delete later
{
	Node current = First();
	int newline = 1;

	printf( "%s\n", comment );

	while ( current )
	{
		printf ("%3d ", attribut[ current ] );

		if ( newline == 10)
		{
			printf( "\n" );
			newline = 0;
		}

		newline++;
		current = Next();
	}

	printf( "\n\n" );
}

