////////////////////////////////////////////////////////////////////////////////
// File         : route.h
// Subsystem    : Graph
// Component    : graphs
// Author       : Josef Huebl, (Josef.Huebl@sss.de)
// Description  : definition of the class route
////////////////////////////////////////////////////////////////////////////////
// Copyright (C) Triple-S GmbH (www.sss.de) 2003
// Alle Rechte vorbehalten. All Rights Reserved.
//
// The reproduction, transmission or use of this document or its contents is
// not permitted without express written authority.
// Offenders will be liable for damages. All rights, including rights created
// by patent grant or registration of a utility model or design, are reserved.
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// History:
// 03.09.03    Josef Huebl       created
// 17.12.03    Josef Huebl       1. released version.
// 30.05.06    Josef Huebl       some extensions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Hints:      
//     
// The class Router implements a router, which solves the colored tree problem.
// For detailed information see the document "Colored Trees.doc".
//     
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_ROUTE_H__9090130D_0D83_4965_9738_9EFE5E41370F__INCLUDED_)
#define AFX_ROUTE_H__9090130D_0D83_4965_9738_9EFE5E41370F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <setjmp.h>
#include "graph.h"

class Router  
{
public:
	Router( const Graph &g, NodeAttribute<int> &color, NodeAttribute<bool> &multible, NodeAttribute< int > &componentNumber, NodeAttribute<int> &id ) : m_graph( g ), m_color( color ), m_multible( multible ), m_componentNumber( componentNumber), m_id( id ) {};
	bool SearchColoredPathes( NodePathVector &pathVector );
	bool CheckPathes( NodeVector &sourceNodes, NodeVector &targetNodes, NodePathVector &pathes ); 

private:
	bool ValidStep( void );
	bool ValidPath( void );
	bool FastCheck(  NodePathVector &pathes );
	void SortPathesToComponents( NodePathVector &pathes );

	const Graph &m_graph;
	NodeAttribute<bool> &m_multible;
	NodeAttribute<int> &m_color;
	NodeAttribute<int> &m_componentNumber;
	NodeAttribute<int> &m_id; // for debugging
	int m_noneColor;
	int m_numberOfPathes;
	int m_currentPathColor;
	int m_currentPathNumber;
	NodePath *m_currentPath;
	long m_mumberOfCombinations;  // for debugging only

	jmp_buf m_jmp_mark;              /* Address for long jump to jump to */

};

#endif // !defined(AFX_ROUTE_H__9090130D_0D83_4965_9738_9EFE5E41370F__INCLUDED_)
