#include <iostream>
#include <string>
#include <deque>
#include "graph.h"
#include "search.h"
using namespace std;

Graph	g;

int main()
{
	Node x, y;
	Node n1 = g.CreateNode();
	Node n2 = g.CreateNode();
	Edge e  = g.CreateEdge(n1, n2);

	NodeAttribute<bool> visited;	// automatisch alle auf false (falls Graph angegeben)
	visited[n1] = true;

	Node n3 = g.CreateNode();
	visited[n3] = true;		// geht auch!
	visited[n2] = true;
	if (visited[n2])
		cout << "n2 visited" << endl;
	else
		cout << "n2 not visited" << endl;

	g.Delete(n2);	// visited[]-ID 1 wird frei
	Node n4 = g.CreateNode();	// visited[]-ID 1 neu vergeben, Wert neu init.
	if (visited[n4])
		cout << "n4 visited" << endl;
	else
		cout << "n4 not visited" << endl;

	EdgeAttribute< string > edgeLabel("foo");	// autom. alle auf "foo"
	e = g.CreateEdge(n3, n4);
	edgeLabel[e] = "bla";
	cout << edgeLabel[e] << endl;
	cout << g.NumberOfNodes() << "/" << g.NumberOfEdges() << endl;

	// Vorbereitung f. Algorithmen auf Graph:
	NodeAttribute< int > nodeID;
	nodeID[n1] = 1;
	nodeID[n3] = 3;
	nodeID[n4] = 4;
	g.CreateEdge(n1, n3);
	g.CreateEdge(n4, n1);
	nodeID[g.CreateNode()] = 9;	// isolierter Knoten

	// DFS:
	GraphSearch s;
	for(s.startDFS(g); !s.done(); s.Next())
		cout << "visiting nodeID " << nodeID[s.n] << endl;

	// topologische Sortierung berechnen:
	deque<Node> topSort2Node;
	for(s.startDFSPost(g); !s.done(); s.Next())
		topSort2Node.push_front(s.n);
	cout << "topologische Sortierung: ";
	for (unsigned int i=0; i<topSort2Node.size(); ++i)
		cout << nodeID[topSort2Node[i]] << " ";
	cout << endl;

	// Knoten markieren, die auf irgendeinem s-t-Pfad liegen
	NodeAttribute< bool > onSTpath;
	for(s.startDFSPost(g, n1); !s.done(); s.Next())
	{
		if (s.e.TargetNode() == n4)
			onSTpath[s.e.TargetNode()] = true;
		else if (s.e)
			onSTpath[s.e.SourceNode()] |= onSTpath[s.e.TargetNode()];
	}

	cout << "g visited: ";

	for (x=g.FirstNode(); x; x=x.NextNode())
	{
		if (visited[x])
			cout << "t";
		else
			cout << "f";
	}

	cout << endl;


	NodeAttribute< bool > kopieVisited( visited ); // kopieVisited ist Attribute von g 

	cout << "g kopieVisited: ";

	for (x=g.FirstNode(); x; x=x.NextNode())
	{
		if (kopieVisited[x])
			cout << "t";
		else
			cout << "f";
	}

	cout << endl;

	
	Graph kopieG(g);

	kopieVisited[ kopieG.FirstNode() ] = visited[ g.FirstNode() ]; // kopieVisited wird Attribute von kopieG

	cout << "kopieG kopieVisited: ";

	for (y=kopieG.FirstNode(); y; y=y.NextNode())
	{
		if (kopieVisited[y])
			cout << "t";
		else
			cout << "f";
	}

	cout << endl;



	if ( kopieVisited == visited)
		cout << "kopieVisited == visited" << endl;
	else
		cout << "kopieVisited != visited" << endl;

	NodeAttribute< bool > kopie2Visited( true ); 

	kopie2Visited = kopieVisited;

	cout << "kopieG kopie2Visited: ";

	for (y=kopieG.FirstNode(); y; y=y.NextNode())
	{
		if (kopie2Visited[y])
			cout << "t";
		else
			cout << "f";
	}

	cout << endl;


	if ( kopie2Visited == kopieVisited)
		cout << "kopie2Visited == kopieVisited" << endl;
	else
		cout << "kopie2Visited != kopieVisited" << endl;

	return 0;
}

