#include <iostream>
#include "route.h"
using namespace std;

#define TRACE

Graph	g;
NodeAttribute<int> id;	
NodeAttribute<int> color ( 0 );	// 0 = no color

void ConvexHull( Graph &g, int numberOfSources, Node *sources, int numberOfTargets, Node *targets )
{
	int i, j;

#ifdef TRACE
			printf( "\nConvexHull\n " );
#endif

	for (i=0; i<numberOfSources; i++)
		for (j=0; j<numberOfTargets; j++)
		{
			g.CreateEdge( sources[i], targets[j] );
#ifdef TRACE
			printf( "(%d,%d) ", id[ sources[i] ], id[ targets[j] ]  );
#endif
		}
}



void Connect( Graph &g, int numberOfNodes, Node *sources,  Node *targets )
{
	int i;

#ifdef TRACE
			printf( "\nConnect\n " );
#endif

	for (i=0; i<numberOfNodes; i++)
	{
		g.CreateEdge( sources[i], targets[i] );
#ifdef TRACE
		printf( "(%d,%d) ", id[ sources[i] ], id[ targets[i] ]  );
#endif
	}
}


void ConnectX( Graph &g, int dist, int numberOfNodes, Node *sources,  Node *targets )
{
	int i;

#ifdef TRACE
			printf( "\nConnectX\n " );
#endif

	for (i=0; i<numberOfNodes; i++)
	{
		g.CreateEdge( sources[i], targets[i] );
		g.CreateEdge( sources[i], targets[ i + dist ] );
		g.CreateEdge( sources[i + dist ], targets[ i + dist ] );
		g.CreateEdge( sources[i + dist], targets[i] );

#ifdef TRACE
		printf( "(%d,%d) ", id[ sources[i] ], id[ targets[i] ]  );
		printf( "(%d,%d) ", id[ sources[i] ], id[ targets[i + dist] ]  );
		printf( "(%d,%d) ", id[ sources[i + dist] ], id[ targets[i + dist] ]  );
		printf( "(%d,%d) ", id[ sources[i + dist] ], id[ targets[i] ]  );
#endif
	}
}

// prermutates the colors
void SwitchColors( int numberOfNodes, Node *nodes )
{
	int i, col;


	for (i=0; i<numberOfNodes/2; i++)
	{
		col = color[ nodes[i] ];
		color[ nodes[ i ] ] = color[ nodes[ numberOfNodes - i - 1 ] ];
		color[ nodes[ numberOfNodes - i - 1 ] ] = col ;
#ifdef TRACE
		printf( "color[ %d ] = %d\n", id[ nodes[i] ], color[ nodes[i] ] );
		printf( "color[ %d ] = %d\n", id[ nodes[numberOfNodes - i - 1] ], color[ nodes[numberOfNodes - i - 1] ] );
#endif
	}
}







int main()
{
	const int NumberOfLevels = 6;
	const int NumberOfNodesPerLevel = 4; // 100;
	const int numberOfSources = NumberOfNodesPerLevel;
	const int numberOfTargets = NumberOfNodesPerLevel;
	const int NumberOfNodes = NumberOfNodesPerLevel * NumberOfLevels;
	const int DistX = NumberOfNodesPerLevel/2;
	const int HullSize2 = NumberOfNodesPerLevel/2;

	Node nodes[ NumberOfNodes ];
	NodeVector sources( numberOfSources );
	NodeVector targets( numberOfTargets );
	NodePathVector pathes( numberOfTargets );

	int i;

	// construct graph
	for ( i=0; i<NumberOfNodes; i++)
	{
		nodes[i] = g.CreateNode();
		id[ nodes[i] ] = i;
	}

	for ( i=0; i<NumberOfLevels-1; i++)
	{
		Connect( g, NumberOfNodesPerLevel, &(nodes[i*NumberOfNodesPerLevel]),  &(nodes[(i+1)*NumberOfNodesPerLevel]) );
	}

	ConnectX( g, DistX, NumberOfNodesPerLevel-DistX, &(nodes[2*NumberOfNodesPerLevel]),  &(nodes[3*NumberOfNodesPerLevel]) );
	ConvexHull( g, HullSize2, &(nodes[1*NumberOfNodesPerLevel]),  NumberOfNodesPerLevel/2, &(nodes[2*NumberOfNodesPerLevel]) );
	ConvexHull( g, HullSize2, &(nodes[3*NumberOfNodesPerLevel]),  NumberOfNodesPerLevel/2, &(nodes[4*NumberOfNodesPerLevel]) );
	ConvexHull( g, HullSize2, &(nodes[1*NumberOfNodesPerLevel+HullSize2]),  HullSize2, &(nodes[2*NumberOfNodesPerLevel+HullSize2]) );
	ConvexHull( g, HullSize2, &(nodes[3*NumberOfNodesPerLevel+HullSize2]),  HullSize2, &(nodes[4*NumberOfNodesPerLevel+HullSize2]) );

	// set SourceNode nodes

	for ( i=0; i<numberOfSources; i++)
	{
		sources[i] = nodes[i];
		color[ sources[i] ] = (i+1)*100;
#ifdef TRACE
		printf( "SourceNode color[ %d ] = %d\n", id[ sources[i] ], color[ sources[i] ] );
#endif
	}

	// set TargetNode nodes

	for ( i=0; i<numberOfTargets; i++)
	{
		targets[i] = nodes[ i + (NumberOfNodes - numberOfTargets) ];
		color[ targets[i] ] = (i+1)*100;
#ifdef TRACE
		printf( "TargetNode color[ %d ] = %d\n", id[ targets[i] ], color[ targets[i] ] );
#endif
	}

	for ( i=0; i<numberOfTargets-DistX; i++)
	{
		color[ targets[i+DistX] ] = color[ targets[i] ];
#ifdef TRACE
		printf( "TargetNode color[ %d ] = %d\n", id[ targets[i+DistX] ], color[ targets[i+DistX] ] );
#endif
	}

	SwitchColors( numberOfSources, &(sources[0]) );
	SwitchColors( HullSize2, &(targets[0]) );
	SwitchColors( HullSize2, &(targets[HullSize2]) );


	// prepare pathes

	for (i =0; i<numberOfTargets; i++ )
	{
		pathes[i].AddLast( targets[i] );
	}

	// search pathes

	NodeAttribute< bool > multible( true ); 
	NodeAttribute< int > componentNumber( 0 );

	Router router( g, color, multible, componentNumber, id );
	bool ret = router.SearchColoredPathes( pathes );

	if (ret)
		printf( "\nSearch SUCCESS\n");
	else
		printf( "\nSearch FAILURE\n");

	// check pathes

	ret = router.CheckPathes( sources, targets, pathes );

	if (ret)
		printf( "\nCheck SUCCESS\n");
	else
		printf( "\nCheck FAILURE\n");


	// print pathes
	for ( i=0; i<numberOfTargets; i++)
	{
		char strNumber[32];
		sprintf( strNumber, "path %d :", i );
		pathes[i].Print( strNumber, id );
	}

	return 0;
}

